// This test fixture generates a video stream in 422 format that the XUP board sees
// on its inputs from the video codec.
// It also captures the output of the video pipeline, the syncs and RGB data to an output file.
// Alternatively, the input data stream can be dumped to a file.

`timescale 1ns/1ns

module wavelet_tf( );

   reg [8:0] colAddrIn;	
   reg [7:0] lineAddrIn;	
   wire [7:0] pixelBlueIn;	
   wire [7:0] pixelGreenIn;	
   wire [7:0] pixelRedIn;		
   wire 		 pixelValidIn;	
   wire 	  enable;
   wire 	  newFrame;

   wire [8:0] colAddrOut;		
   wire [7:0] lineAddrOut;	
   wire [7:0] pixelBlueOut;	
   wire [7:0] pixelGreenOut;	
   wire [7:0] pixelRedOut;	
   wire 	  pixelValidOut;	
   
   wire clk, reset;
   
   clockGen clock_inst
	 (.clk				(clk),
	  .reset			(reset));
   
   filter f
	 (.pixelValidOut	(pixelValidOut),
	  .lineAddrOut		(lineAddrOut[7:0]),
	  .colAddrOut		(colAddrOut[8:0]),
	  .pixelRedOut		(pixelRedOut[7:0]),
	  .pixelGreenOut	(pixelGreenOut[7:0]),
	  .pixelBlueOut	(pixelBlueOut[7:0]),

	  .clk				(clk),
	  .enable			(enable),
	  .newFrame		(newFrame),
	  .pixelValidIn	(pixelValidIn),
	  .lineAddrIn		(lineAddrIn[7:0]),
	  .colAddrIn		(colAddrIn[8:0]),
	  .pixelRedIn		(pixelRedIn[7:0]),
	  .pixelGreenIn	(pixelGreenIn[7:0]),
	  .pixelBlueIn		(pixelBlueIn[7:0]));
   
   localparam NumLines = 240, NumCols = 320;
   parameter 	inFileName = "imageIn.dat";
   parameter 	outFileName = "imageOut.dat";

   reg [23:0]  imageMemIn[0:NumLines-1][0:NumCols-1];
   reg [23:0]  imageMemOut[0:NumLines-1][0:NumCols-1];
  
   initial begin
      $display("Loading data from %s", inFileName);
      $readmemh(inFileName, imageMemIn);
   end
   
   integer count;
   localparam IDLECOUNT = 5;
   localparam VALID = 1, NOTVALID = 0, RESTING = 2;
   reg [1:0] state;

   assign enable = 1;      // Set on/off for debugging
   assign newFrame = 0;    // Not used
   assign pixelValidIn = state[0];
   assign pixelRedIn = imageMemIn[lineAddrIn][colAddrIn][23:16];
   assign pixelGreenIn = imageMemIn[lineAddrIn][colAddrIn][15:8];
   assign pixelBlueIn = imageMemIn[lineAddrIn][colAddrIn][7:0];

   always @(posedge clk) begin
      if (reset) begin
		 lineAddrIn <= 0;
		 colAddrIn <= 0;
		 state <= NOTVALID;
      end else begin
		 case (state) 
		   NOTVALID: begin
			  state <= VALID;
		   end
		   VALID: begin
			  if (colAddrIn == NumCols-1) begin
				 colAddrIn <= 0;
				 lineAddrIn <= lineAddrIn + 1;
				 count <= 0;
				 state <= RESTING;
			  end else begin
				 colAddrIn <= colAddrIn + 1;
				 state <= NOTVALID;
			  end
		   end
		   RESTING: begin
			  if (count == IDLECOUNT) begin
				 state <= VALID;
			  end else begin
				 count <= count + 1;
			  end
		   end
		 endcase // case (state)
	  end // else: !if(reset)
	  if (pixelValidOut) begin
		 imageMemOut[lineAddrOut][colAddrOut] <= { pixelRedOut, pixelGreenOut, pixelBlueOut };
	  end
	  if (lineAddrIn == NumLines && state == VALID) begin
		 $display("Writing to output image file %s.",outFileName);
		 $writememh(outFileName, imageMemOut);
		 $stop;
	  end
   end // always @ (posedge clk)

endmodule // wavelet_tf



